#ifndef ___STOCHASTIC_MAPPING
#define ___STOCHASTIC_MAPPING

#include "definitions.h"
#include "tree.h"
#include "stochasticProcess.h"
#include "sequenceContainer.h"
#include "suffStatComponent.h"

typedef vector<pair<int, MDOUBLE> > mutationHistoryPerBranch; //mutations history along a single branch. first member in the pair is the state id. Second member is the amount of time spent in that state
typedef vector<mutationHistoryPerBranch > treeMutationHistory; //the mutations along the whole tree

class stochasticMapping {
public:
	explicit stochasticMapping (const tree& inTree, const stochasticProcess& sp);
	
	//simulate mutations on the phylogeny a given number of times so the resulting leaves are identical to the input data, 
	//as given in the specified position of the sequenceContainer. 
	//The result is stored as the history of mutations for each branch.
	//retruns false if fails to generate mapping for the specified data
	bool generateStochasticMapping(const sequenceContainer& inSc, int pos, int iterations, vector<treeMutationHistory >& res);
	static vector<treeMutationHistory> getMappingForEachTree(const vector<tree>& trees, const stochasticProcess* pCharModel, const sequenceContainer& inSc, int pos);
	
	virtual ~stochasticMapping();
	sequenceContainer toSeqData();
	sequenceContainer toSeqDataWithoutInternalNodes();
	Vint getNodeContents() const {return _nodesContent;}


	
private:
	//generate mapping given the pre-calculated statistics. Store results in _nodesContent
	void sampleAncestrals(const Vdouble& rootProbs, const VVVdouble& samplingProbs);
	Vdouble getRootProbs(const suffStatGlobalHomPos& sscUp) const;
	VVVdouble getSamplingProbs(const suffStatGlobalHomPos& sscUp) const;
	//given sampled ancestrals and tips data: simulate mutations on phylogeny. Add result to res.
	void sampleMutationsGivenAncestrals(treeMutationHistory& res);
	mutationHistoryPerBranch sampleMutationsGivenAncestralsPerBranch(int fatherState, int sonState, MDOUBLE branchLength);
	
	void generateRootSeq(int seqLength);	
	void init();
	
	vector<sequence> _simulatedSequences; // the sequences (nodes * seqLen)
	Vdouble _waitingTimeParams;//each entry is the lambda parameter of the exponential distribution modeling the waiting time for "getting out" of state i
	//_jumpProbs[i][j] is the probability of jumping from state i to state j (given that a change has ocured).
	VVdouble _jumpProbs; 

	tree _et;
	const stochasticProcess& _sp;
	sequenceContainer _sc;
	Vint _nodesContent; // the actual state at each node, retrieval according to node id
};

#endif

