#ifndef __TRAIT_RATE_OPTIONS
#define __TRAIT_RATE_OPTIONS

#include "definitions.h"
#include "twoStateModel.h"
//#include "charAndSeqModel.h"
#include <string>
#include <fstream>
using namespace std;


enum sequenceType {CODING, NON_CODING, PROTEIN, NON_CODING_3}; //CODING=coding nucleotide: use codon model, NON_CODING = non coding nucleotide, PROTEIN = amino acid sequence, NON_CODING_3: perform calculation on third positions nucleotides. 
enum seqModelType {JC, K2, HKY, GTR, JTT, MT_REV, CP_REV, LG, WAG, AA_JC}; //for non coding: JC = Jukes-Cantor 1 param, K2 = Kimura 2 parameters, HKY: K2+base freqs, GTR=general time reversible. teh rest are aa models
enum combinedModelType {M0, M1}; //M0 = no rate difference between the binary characters. LL is computed based solely on the sequence data. M1= combined model
enum llCalc {SEQ, COMBINED};
class traitRateOptions {
		
public:
    static void initOptions(const string& paramFileName);
	virtual ~traitRateOptions();

	static string getRootFreqTypeStr(twoStateModel::rootFreqType type);
    static twoStateModel::rootFreqType getRootFreqTypeFromStr(const string& str);

	static string getSequenceTypeStr(sequenceType type);
    static sequenceType getSequenceTypeFromStr(const string& str);

	static string getseqModelTypeStr(seqModelType type);
    static seqModelType getseqModelTypeFromStr(const string& str);

	static string getCombinedModelTypeStr(combinedModelType type);
    static combinedModelType getCombinedModelTypeFromStr(const string& str);

	static string getLlCalcTypeStr(llCalc type);
    static llCalc getLlCalcTypeFromStr(const string& str);
	

	
private:
	static void initDefault();
	static void getParamsFromFile(const string& paramFileName);

public:
	static string _mainType;
	static string _treeFile;
	static string _treesFile; //to handle more than one possible tree input
	static string _characterFile;
	static string _seqFile;
	static string _logFile;
    static int _logValue; 
	static string _outDir;
	static string _outFile;
	static string _scaledTreeFile;

	static twoStateModel::rootFreqType _rootFreqType;
	static combinedModelType _modelType;
	static llCalc _llCalcType;
	
	static string _rootAt;
	
	static MDOUBLE _relRate; //the relative rate of state 1 relative to state 0
	//character model parameters:
	static MDOUBLE _charModelParam1;
	static MDOUBLE _charModelParam2;
	//molecular model parameters:
	static MDOUBLE _globalRate; //parameter to scale the branches of the input tree
	static MDOUBLE _seqModelParam1; //for HKY+GTR
	static MDOUBLE _seqModelParam2; //for GTR (+timura92 if implemented)
	static MDOUBLE _seqModelParam3; //for GTR only
	static MDOUBLE _seqModelParam4; //for GTR only
	static MDOUBLE _seqModelParam5; //for GTR only
	static MDOUBLE _seqModelParam6; //for GTR only
	static int _gammaCategories; //if =1 then homogenous rate distribution 
	static MDOUBLE _gammaParam; //the alpha rate parameter for the gamma distribution

	static MDOUBLE _treeLength; //if > 0: keep all scaled tree to a specified tree length
	static string _referenceTree; // if given - set the total tree length according to the total length of the ref tree

	//static 
	//stochastic mapping params
	static int _stochasicMappingIterations;//number of simulated stochastic mapping iterations
	
	//optimization params
	static bool _bScaleTree; 
	static bool _bOptRelRate; 
	static bool _bOptSeqModel; 
	static bool _bOptCharModel; 
	static int _maxOptimizationIterations; //the total number optimization iterations. Will not be changed by users
	static MDOUBLE _epsilonLLimprovement; //if the log-likelihood after optimization is lower than this threshold - then optimize again. Will not be changed by users
	static MDOUBLE _tolParamOptimization; 
	//parameters for optimize many starts
	//the input to these paraqmeters in the param fule is a string of a list seperated by commas ("10,2,1")
	static Vint _optimizePointsNum; //a vector with the number of points to peformed the current cycle of optimization. 
	static Vint _optimizeIterNum; //the number of iterations to perform in each cycle.
	static Vint _optimizeStartegies; //if==0: full likelihood computaion. if==1: compute one tree based on all stochastic mappings. If>1: perform k-mean clustering to group similar trees

	//simulation options
	static MDOUBLE _initTreeLength;//the total length of the simulated tree. 0.0: don't scale. positive(1.5): indicate total length. Negative(-1.5): the average length from each leaf to the root. We assume the tree is ultrametric
	static string _simulationsTreeDir;//specify the folder where all the simulated trees are
	static int _simulationSeqPos;//the number of nucleotides to simulate
	static int _startSimulationsIter; //when simulating random trees - simulate tree numbers from _startSimulationsIter to  _endSimulationsIter.
	static int _endSimulationsIter;
	
	//RRtree options
	static sequenceType _sequenceType;
	static seqModelType _seqModelType;
};

#endif
